=begin
#
TIG (c) 2005 - 2013
All rights reserved.
#
Name : SectionCutFace.rb
Type : Context Menu Tool == "Add Section-Cut Face"
Description : Adds a Group of Faces to a Selected Section-Plane
#
Note: it is available only for SUp v8 or newer, and runs only 
from the Context-Menu, and if selection is a Section-Plane.
#

A Dialog asks for the section-cut's 'Material'.
Choose from the dropdown list, which consists of:

<Default>
<No_Faces>
Black [the initial default]
White
Red
Orange
Yellow
Green
Blue
Indigo
Violet
Cyan
Magenta
Gray
Brown
<then all other materials in the model by sorted name>

To scroll through a potentially long list of 'Materials'  use 
the up/down-arrow keys, type in first letter of a color to jump to it 
- e.g. W --> Wheat then use the up/down arrow keys to move up/down 
from that for Wombat.
<Default> == No material.
<No_Face> == Remove all Faces from the cut, leaving only Edges - 
with this option these edges can NEVER be 'Hidden'. otherwise you'd have 
nothing visible in the section-cut group.
You can Glue/On_Demand a <No_Face> version but remember that no faces 
will be visible when the section-cut group is located on the Plane...
The chosen color/material is added to every face/backface in the 
section-cut's group.
You can edit the faces and/or materials within a section-cut group, 
BUT if it is set as Glued/On_Demand then it will subsequently update 
and your changes will be lost: so it's best only to edit non-Glued cuts.
A cut's 'facing' is determined from its nested outlines: on solids this 
will work as expected, but interpenetrating solids or badly formed 3d 
objects can result in some voids where solids are preferred etc - then 
editing will be needed.  This need to be a non-Glued version to retain 
your changes permanently.
To make a non-Glued version select the Glued/On_Demand version, unlock 
it [if necessary] and press <delete>, then remake the cut, this time 
choosing 'No' for the 'Glue' option - See below...
#
Each Section-Plane's 'cut' edge-color & width are set by the Scene's 
'Style'.
You may wish to match those with the faces' color, with a thin 'width'.
#
It asks if you want to 'Keep' faceless edges [default=Yes], most useful 
with the <No_Face> option when making an outline etc.  Obviously you 
cannot erase faceless edges if the <No_Face> option is chosen - otherwise 
the cut will be empty.
#
It asks if you want to 'Hide' edges [default=Yes] - see earlier about 
its limitations - you cannot hide edges in <No_Face> cuts, otherwise 
the cut will look empty.
#
It asks if you want to 'Lock' the new section-cut-face group 
[default=Yes]: this stops it being moved/changed accidentally.
#
It asks if you want to 'Glue' the section-cut-face to the Section-Plane 
[default=No]: 
'Yes' means it will change as the Section-Plane or its cutting of the 
model changes.  
'No' leaves it where/as as it was first made - this is the best if you 
intend editing the cut's contents. 
'On_Demand' does NOT Glue the cut, BUT the additional context-menu item 
available if the Section-Plane has this setting ['Redo SectionCutFace'] 
will redo the cut - without a dialog, using the previous settings - useful 
in complex models where refreshing cuts continuously would be too onerous.
#
You can make any number of cuts using No and moving the Plane.
But only one Glued/On_Demand cut can exist for a Plane, change Plane's 
cut from on type to another will remove the previous linking.
#
It asks for the section-cut group's 'Layer' - this default is shown 
first 'SCUT-nnnnnnn' - where nnnnnnn = 'a time stap'.  It's best to 
leave each one on its own layer so you can switch it off if the 
section-cut is not wanted 'on' in some scene views...
The layers and section-plane width/colors etc are set scene by scene.
The cut's faces & edges are on 'Layer0' and are inside a group 
called 'SCUT-nnnnnnn', it's put on its own own unique layer UNLESS 
you choose otherwise. 
These default layers are 'on' only in the current scene by default: 
switch on/off to suit other pages.
#
The section-cut group is added ~0.2mm inside the cut plane so it's 
then visible in the section-plane's cut.

If the edges of the cut-group are set to be 'hidden' - to see/edit 
them switch on View > Hidden Geometry.

The cut-group is readily editable [but see earlier about 'Glued' 
cuts updating] - if edge editing when lines are hidden then you must 
have 'view > hidden geometry' switched 'on' - double-click group to 
start editting and when done close it.

Complex models might take a few seconds to complete a cut.
Consider this if 'Gluing', as delays every time you modify the model 
might be annoying... In this scenario 'On_Demand' might be the more 
appropriate option, or even 'No'...

The newly added cut-face/layer is also easily undo-able in one step.

Please save your model before using this tool, BugSplats rarely happen!

This file registers the Extension from the subfolder of the same name.

The tool's '.strings' file can be used to make locale translations...

Version:
3.0 20130606
Recoded. Speed improved. Cut-face can now 'Glue' to the SectionPlane 
so that then it changes with it/model.
3.1 20120606
Fixed glitch with cut-faces creation for hidden objects.
3.2 20130607
Fixed glitch with glued cut-faces not updating in convoluted nested contexts.
3.3 20130607
SectionCutFace_loader.rb file now contains editable lists for changing default 
colors and the dialog's initial default values.
3.4 20130607
Glue 'On_Demand' option added to avoid continual dynamic SCUT updating.  
The initial Glue default is now 'No'.
SectionCutFace_loader.rb customized options now work properly in compiled files.
3.5 20130608
Option permutations allow more flexibility, <No_Face> can be Glued/On_Demand etc.
3.6 20130609
Recoded legit group manipulations to avoid bugsplats from ill-advised 3rd-party 
tools over-writing API methods and/or adding badly formed Observers.
3.7 20130611
SectionPlanes 'on-face' now cut geometry by making the cut 0.1mm inside section.
Option in SectionCutFace_loader.rb to change 'Inset' to > 0.1.mm, needed for 
some users whose GC's leave section-cuts not showing...
3.8 20120724
Glitch with observers used for Gluing when new model started resolved.
=end
###
require 'sketchup.rb'
require 'extensions.rb'
###
module TIG
###
module SectionCutFace
###
### Setup Extn Constants
VERSION="3.8"
NAME="SectionCutFace"
CREATOR="TIG"
COPYRIGHT="#{NAME} © #{Time.now.year}"
### Setup Dir Constants
FOLDER=File.join(File.dirname(__FILE__), NAME)
STRINGS=File.join(FOLDER, "Strings")
### Setup translated Constants
### File.exists? should always work as all ASCII in path?
	file=File.join(STRINGS, "SectionCutFace-#{Sketchup.get_locale.upcase}.strings")
	if File.exist?(file)
		lines=IO.readlines(file)
	else
		file=File.join(STRINGS, "#{NAME}-EN-US.strings")
		if File.exist?(file)
			lines=IO.readlines(file)
		else
			lines=[]
		end
	end
	lines.each{|line|
		line.chomp!
		next if line.empty? || line=~/^[#]/
		next unless line=~/[=]/
		eval(line) ### set CONSTANT=string
	}
###
### Extension set up
EXT=SketchupExtension.new(NAME, File.join(FOLDER, "SectionCutFace_loader.rb"))
EXT.name = NAME
EXT.description = DESC
EXT.version = VERSION
EXT.creator = CREATOR
EXT.copyright = COPYRIGHT
Sketchup.register_extension(EXT, true) # show on 1st install
###
end #module SectionCutFace
###
end #module TIG
###
